@@ -158,7 +158,7 @@ do_umount( const char* device, int do_lazy )
 int
 main( int argc, char** argv )
 {
-    char device[PATH_MAX], mntptdev[PATH_MAX];
+    char device[PATH_MAX], mntptdev[PATH_MAX], path[PATH_MAX];
     const char* fstab_device;
     int is_real_path = 0;
     int do_lazy = 0;
@@ -210,9 +210,18 @@ main( int argc, char** argv )
     }
 
     /* if we got a mount point, convert it to a device */
+    debug ("checking whether %s is a mounted directory\n", argv[optind]);
     if( fstab_has_mntpt( "/proc/mounts", argv[optind], mntptdev, sizeof(mntptdev) ) ) {
         debug( "resolved mount point %s to device %s\n", argv[optind], mntptdev );
         argv[optind] = mntptdev;
+    } else if( !strchr( argv[optind], '/' ) ) {
+        /* try to prepend MEDIADIR */
+        snprintf( path, sizeof( path ), "%s%s", MEDIADIR, argv[optind] );
+        debug ("checking whether %s is a mounted directory\n", path);
+        if( fstab_has_mntpt( "/proc/mounts", path, mntptdev, sizeof(mntptdev) ) ) {
+            debug( "resolved mount point %s to device %s\n", path, mntptdev );
+            argv[optind] = mntptdev;
+        }
     }
 
     /* get real path, if possible */
